use utf8;
use strict;

# サクラエディタ用キーワードファイル作成スクリプト
# 本体ver.2.26のタグデータベースに対応。他のバージョンは未確認。

# configure
my $ColTag     = 'RK1';	# KAG3 tag
my $ColSpecial = 'RK2';	# KAG3 special tag
my $ColAttr    = 'RK3';	# attribute
my $ColLabel   = 'RK4';	# label
my $ColUser    = 'RK5';	# user macro
my $ColComment = 'CMT';	# KAG comment

my $Infile  = 'tags.database.tml';	# database file
my $Outfile = 'kag3.rkw';	# output file

my $ThresStrLen = 48;

# const.
my @SpecialTagsBase = qw(
if endif ignore endignore iscript endscript 
macro endmacro macropop erasemacro emb jump call return
);

my @ExtraTags = qw(
macropop
);

my %SpecialTags = map { ($_, 1); } (@SpecialTagsBase);

my @out = ();
my @tags = ();
my $cnt = 0;

# read DB
open (FI, $Infile) or die("$Infile: $!");
while (<FI>) {
	if (/<tag\s+name="(\w+)">/) {
		my $tag = $1;
		push @tags, $tag;
	}
}
close (FI);

push @tags, @ExtraTags;

# tags
my @ntags = ();
my @stags = ();
map { @SpecialTags{$_} ? (push(@stags, $_)) : (push(@ntags, $_)) } (sort { $b cmp $a } @tags);
$cnt = pushBuffer(\@out, \@stags, $ColSpecial, $cnt);
$cnt = pushBuffer(\@out, \@ntags, $ColTag, $cnt);

# misc.
push @out, sprintf(qq!RxKey[%03d]=${ColUser},/((^\@)|\\[)\\w+\\b/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColTag},/\\]/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColAttr},/\\w+=(['"]).*?\\1/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColAttr},/\\w+=\\S+/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColLabel},/^\\*[^|]+\\|.*/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColLabel},/^\\*[^|]+\\|?/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColLabel},/^\\*\\|.*/k!, $cnt++);
push @out, sprintf(qq!RxKey[%03d]=${ColComment},/^;.*/k!, $cnt++);

# output
open (FO, ">$Outfile") or die("$Outfile: $!");
foreach my $k (@out) {
	print FO "$k\n";
}
close (FO);


sub getTagRegex
{
	my ($tags, $col, $cnt) = @_;
	my $rv = sprintf("RxKey[%03d]=${col},/((^\@)|\\[)(${tags})\\b/k", $cnt);
	return $rv;
}

# なるべく "|" でくっつけるように。
sub pushBuffer
{
	my ($out, $xtags, $col, $cnt) = @_;
	my $len = 0;
	my @wtags = ();
	foreach my $tag (@$xtags) {
		$len += length($tag) + 1;
		if ($len > $ThresStrLen) {
			my $w = join('|', @wtags);
			push @$out, getTagRegex($w, $col, $cnt++) if ($w);
			@wtags = ();
			$len = 0;
		}
		push @wtags, $tag;
	}
	# 後始末
	my $w = join('|', @wtags);
	push @$out, getTagRegex($w, $col, $cnt++) if ($w);

	return $cnt;
}

